import { ConfigSpec } from './ConfigSpec';
import { EvaluationReason } from './EvaluationReason';
import { ExplicitStatsigOptions } from './StatsigOptions';
import { IDList } from './utils/IDListUtil';
import StatsigFetcher from './utils/StatsigFetcher';
export declare type ConfigStore = {
    gates: Record<string, ConfigSpec>;
    configs: Record<string, ConfigSpec>;
    idLists: Record<string, IDList>;
    layers: Record<string, ConfigSpec>;
    experimentToLayer: Record<string, string>;
};
export declare type APIEntityNames = {
    gates: string[];
    configs: string[];
};
export default class SpecStore {
    private initReason;
    private rulesUpdatedCallback;
    private initialUpdateTime;
    private lastUpdateTime;
    private store;
    private rulesetsSyncInterval;
    private idListsSyncInterval;
    private disableRulesetsSync;
    private disableIdListsSync;
    private initialized;
    private rulesetsSyncTimer;
    private idListsSyncTimer;
    private rulesetsSyncTimerLastActiveTime;
    private idListsSyncTimerLastActiveTime;
    private rulesetsSyncPromise;
    private idListsSyncPromise;
    private fetcher;
    private dataAdapter;
    private rulesetsSyncFailureCount;
    private idListsSyncFailureCount;
    private getIDListCallCount;
    private bootstrapValues;
    private initStrategyForIDLists;
    private clientSDKKeyToAppMap;
    private hashedClientSDKKeyToAppMap;
    private hashedSDKKeysToEntities;
    constructor(fetcher: StatsigFetcher, options: ExplicitStatsigOptions);
    getInitReason(): EvaluationReason;
    getInitialUpdateTime(): number;
    getLastUpdateTime(): number;
    getGate(gateName: string): ConfigSpec | null;
    getConfig(configName: string): ConfigSpec | null;
    getLayer(layerName: string): ConfigSpec | null;
    getExperimentLayer(experimentName: string): string | null;
    getIDList(listName: string): IDList | null;
    getAllGates(): Record<string, ConfigSpec>;
    getAllConfigs(): Record<string, ConfigSpec>;
    getAllLayers(): Record<string, ConfigSpec>;
    getClientKeyToAppMap(): Record<string, string>;
    getHashedClientKeyToAppMap(): Record<string, string>;
    getHashedSDKKeysToEntities(): Record<string, APIEntityNames>;
    init(): Promise<void>;
    private _initIDLists;
    resetSyncTimerIfExited(): Error | null;
    isServingChecks(): boolean;
    private _fetchConfigSpecsFromServer;
    private _fetchConfigSpecsFromAdapter;
    private _saveConfigSpecsToAdapter;
    private pollForUpdates;
    private logDiagnostics;
    syncConfigSpecs(): Promise<void>;
    syncIdLists(): Promise<void>;
    private _process;
    /**
     * Returns a reverse mapping of layers to experiment (or vice versa)
     */
    private _reverseLayerExperimentMapping;
    private syncIdListsFromDataAdapter;
    private syncIdListsFromNetwork;
    private genFetchIDList;
    shutdown(): void;
    shutdownAsync(): Promise<void>;
    private clearRulesetsSyncTimer;
    private clearIdListsSyncTimer;
    private clearTimers;
    private setInitialUpdateTime;
}
