import { IDataAdapter } from './interfaces/IDataAdapter';
export declare type RulesUpdatedCallback = (rulesJSON: string, time: number) => void;
export declare type RetryBackoffFunc = (retriesRemaining: number) => number;
export declare type StatsigEnvironment = {
    tier?: 'production' | 'staging' | 'development' | string;
    [key: string]: string | undefined;
};
export declare type InitStrategy = 'await' | 'lazy' | 'none';
export interface LoggerInterface {
    debug?(message?: any, ...optionalParams: any[]): void;
    info?(message?: any, ...optionalParams: any[]): void;
    warn(message?: any, ...optionalParams: any[]): void;
    error(message?: any, ...optionalParams: any[]): void;
    logLevel: 'none' | 'debug' | 'info' | 'warn' | 'error';
}
export declare type ExplicitStatsigOptions = {
    api: string;
    apiForDownloadConfigSpecs: string;
    bootstrapValues: string | null;
    environment: StatsigEnvironment | null;
    rulesUpdatedCallback: RulesUpdatedCallback | null;
    logger: LoggerInterface;
    localMode: boolean;
    initTimeoutMs: number;
    dataAdapter: IDataAdapter | null;
    rulesetsSyncIntervalMs: number;
    idListsSyncIntervalMs: number;
    loggingIntervalMs: number;
    loggingMaxBufferSize: number;
    disableDiagnostics: boolean;
    initStrategyForIP3Country: InitStrategy;
    initStrategyForIDLists: InitStrategy;
    postLogsRetryLimit: number;
    postLogsRetryBackoff: RetryBackoffFunc | number;
    disableRulesetsSync: boolean;
    disableIdListsSync: boolean;
    disableAllLogging: boolean;
};
/**
 * An object of properties for initializing the sdk with advanced options
 */
export declare type StatsigOptions = Partial<ExplicitStatsigOptions>;
export declare function OptionsWithDefaults(opts: StatsigOptions): ExplicitStatsigOptions;
export declare function OptionsLoggingCopy(options: Record<string, unknown>): StatsigOptions;
